function dx = hypSys_no_obsv_odefun(t, x, ctrl_func, hypSys)
    
    M           = hypSys.M;
    N           = hypSys.N;
    N_grid      = hypSys.N_grid;
    N_tr        = hypSys.N_tr;
    Delta_tr    = hypSys.Delta_tr;
    
    C           = hypSys.C;
    q           = hypSys.q;
    
    ctrl_on_limit = hypSys.ctrl_on_limit;
    

    %% Extract variables
    u = x(1:N);
    v = x((N + 1):(2*N));
    X = x((2*N + 1):(2*N + M));
    
    z = x((2*N + M + 1):(2*N + M + N_tr));
    
    %% Augment variables
        % System variables
    v0 = 2*v(1) - v(2); % Interpolate v to find v0
    u0 = q*v0 + C*X;    % Find the u0 by using the BC
    u1 = 2*u(N) - u(N - 1); % Interpolate u to find u1
    v1 = 2*v(N) - v(N - 1); % Interpolate v to find v1
    
    u_aug = [u0; u; u1];
    v_aug = [v0; v; v1];
    
        % Transmission line variables
    z0 = 2*z(1) - z(2); % Interpolate z to find z0. This is a somewhat 
                        %   serious hack, as this should have been U(t).
                        %   this is ignored for simplicity
    z1 = 2*z(N_tr) - z(N_tr - 1); % Interpolate v to find v1
    
    z_aug = [z0; z; z1];

    %% Calculate controller input
    if (t >= ctrl_on_limit)
        Uin = ctrl_func(t, u_aug, v_aug, z_aug, X, hypSys);
    else
        Uin = 0;
    end;
    
    %% Calculate derivatives
        % Hyperbolic system
        v_aug(N_grid) = Uin;
        [du, dv, dU] = hyp_sys_dx(t, u_aug, v_aug, X, hypSys);
    
        % Transmission line
        if (t >= ctrl_on_limit) 
            z_aug(1) = Uin;
        else
            z_aug(1) = 0.1*sin(t); % Insert something random
        end;
        dz = -(z_aug(2:(N_tr + 1)) - z_aug(1:(N_tr))) / (Delta_tr); %(= -z_x)
    
    %% Create state vector
    dx = [  du; dv; dU; ...
            dz];