function dx = hypSys_w_redCtrl(t, x, hypSys)
    
    M           = hypSys.M;
    N           = hypSys.N;
    N_grid      = hypSys.N_grid;
    N_tr        = hypSys.N_tr;
    Delta_tr    = hypSys.Delta_tr;
    
    C           = hypSys.C;
    q           = hypSys.q;
    
    ctrl_on_limit = hypSys.ctrl_on_limit;
    obsv_on_limit = hypSys.obsv_on_limit;

    %% Extract variables
    u = x(1:N);
    v = x((N + 1):(2*N));
    X = x((2*N + 1):(2*N + M));
    
    u_hat = x((2*N + M + 1):(3*N + M));
    v_hat = x((3*N + M + 1):(4*N + M));
    X_hat = x((4*N + M + 1):(4*N + 2*M));

    z = x((4*N + 2*M + 1):(4*N + 2*M + N_tr));
    
    %% Augment variables
        % System variables
    v0 = 2*v(1) - v(2); % Interpolate v to find v0
    u0 = q*v0 + C*X;    % Find the u0 by using the BC
    u1 = 2*u(N) - u(N - 1); % Interpolate u to find u1
    v1 = 2*v(N) - v(N - 1); % Interpolate v to find v1
    
    u_aug = [u0; u; u1];
    v_aug = [v0; v; v1];
   
        % Observer variables
    v_hat0 = 2*v_hat(1) - v_hat(2); % Interpolate v_hat to find v_hat0
    u_hat0 = q*v_hat0 + C*X_hat;    % Find u_hat0 by using the BC
    u_hat1 = 2*u_hat(N) - u_hat(N - 1); % Interpolate u_hat to find u_hat1
    v_hat1 = 2*v_hat(N) - v_hat(N - 1); % Interpolate v_hat to find v_hat1
    
    u_hat_aug = [u_hat0; u_hat; u_hat1];
    v_hat_aug = [v_hat0; v_hat; v_hat1];
    
        % Transmission line variables
    z0 = 2*z(1) - z(2); % Interpolate z to find z0. This is a somewhat 
                        %   serious hack, as this should have been U(t).
                        %   this is ignored for simplicity
    zL = 2*z(N_tr) - z(N_tr - 1); % Interpolate v to find v1
    
%     z_aug = [z0; z; zL];

    %% Calculate controller input
    if (t >= ctrl_on_limit)
%         Uin = ctrl_func(t, u_hat_aug, v_hat_aug, z_aug, X_hat, hypSys);
%         Uin = hypSys.C_red * z;

          C_red = hypSys.C_red;
          D_red = hypSys.D_red;
          Uin = C_red * z(1:N_tr) + D_red * u1;
    else
        Uin = 0;
    end;
    
    
    %% Calculate derivatives
        % Hyperbolic system
        v_aug(N_grid) = Uin;
        [du, dv, dX] = hyp_sys_dx(t, u_aug, v_aug, X, hypSys);
        
        % Hyperbolic observer system
        v_hat_aug(N_grid) = Uin;
        if (t >= obsv_on_limit)
            [du_hat, dv_hat, dX_hat] = hyp_sys_obsv_dx(t, u_hat_aug, v_hat_aug, X_hat, u1, hypSys);
        else
            du_hat = zeros(N, 1);
            dv_hat = zeros(N, 1);
            dX_hat = zeros(M, 1);
        end;
    
        % red_obsv line
        A_red = hypSys.A_red;
        B_red = hypSys.B_red;
        
        
        if (t >= ctrl_on_limit)
            red1 = z(1:N_tr);
            dred1 = A_red  * red1 + B_red  * u1;
        else
            dred1 = zeros(N_tr, 1);
        end;
    
    %% Create state vector
    dx = [  du; dv; dX; ...
            du_hat; dv_hat; dX_hat;
            dred1];