function [du, dv, dX] = hyp_sys_dx(t, u_aug, v_aug, X, hypSys)
    %% Calculates the time derivatives for an hyperbolic system on the form
    %   u_t(x, t) = -epsi*u_x(x, t) + a_0 * exp( gamma*x) * v(x, t)
    %   v_t(x, t) =  epsi*v_x(x, t) + b_0 * exp(-gamma*x) * u(x, t)
    %   
    %  with BCs:
    %   u(0, t) = q v(0, t) + C X(t)  
    %   v(1, t) = v_aug(end)
    %   X'(t)  = A X(t)
    
    %% Variable assignments
    A           = hypSys.A;
    N           = hypSys.N;
  	Delta_grid  = hypSys.Delta_grid;
    epsi        = hypSys.epsi;
    a_0         = hypSys.a_0;
    b_0         = hypSys.b_0;
    gamma       = hypSys.gamma;
    
    %% Calculating the state space derivatives
        %States
    u_x = (u_aug(2:(N+1)) - u_aug(1:N)) / Delta_grid;
    v_x = (v_aug(3:(N+2)) - v_aug(2:(N+1))) / Delta_grid;

    xspan = (Delta_grid:Delta_grid:(1 - Delta_grid))';
    du = -epsi * u_x + a_0*exp( gamma.*xspan).*v_aug(2:(N+1));
    dv =  epsi * v_x + b_0*exp(-gamma.*xspan).*u_aug(2:(N+1));
    
        % Disturbance
    dX = A * X;