function h_YU_psf = h_YU_psf_sampling(s_vec, hypSys)

% alpha_LG = 0.1;
% gamma_LG = 2 * alpha_LG;
% numFreqs = 100;
% 
% % s_vec = Laguerre_sampling_points(numFreqs, alpha_LG, gamma_LG);
% s_vec = 1j * logspace(-2, 1, numFreqs);
% % s_vec = 0;
% hypSys = setup_hypSys(2000);

    N_grid = hypSys.N_grid;
    Delta_grid = hypSys.Delta_grid;
    
    r = hypSys.r;
    q = hypSys.q;
    x_bar = hypSys.x_bar;
    a_0 = hypSys.a_0;
    b_0 = hypSys.b_0;
    epsi = hypSys.epsi;
    gamma = hypSys.gamma;
    A = hypSys.A;
    C = hypSys.C;
    L = hypSys.obsv_params.L;
    
    p1 = hypSys.obsv_params.p1;
    p2 = hypSys.obsv_params.p2;

    K_psf_x_bar = hypSys.ctrl_params.K_psf_x_bar;

    x_bar_ind = 1 + ceil(x_bar / Delta_grid);
    
    Kuu = hypSys.Kuu;
    Kuv = hypSys.Kuv;
    Kvu = hypSys.Kvu;
    Kvv = hypSys.Kvv;

    Kvu_x_bar = Kvu(:, x_bar_ind);
    Kvv_x_bar = Kvv(:, x_bar_ind);

    La_x_bar = hypSys.La(1:x_bar_ind, x_bar_ind);
    Lb_x_bar = hypSys.Lb(1:x_bar_ind, x_bar_ind);

    xspan = (0:Delta_grid:1)';
    xispan = (0:(x_bar_ind - 1))' * Delta_grid;


    s_num = length(s_vec);
    
    h_YU_psf = zeros(s_num, 1);

    V = [-1 0; 0 1];
    mu = [1; -q];

    Upsilon = zeros(2, N_grid);
    for k = 0:(N_grid - 1)
        Upsilon(:, k + 1) = (1 / epsi) * [p1(k + 1); -p2(k + 1)];
    end;

    for s_cnt = 0:(s_num - 1)
        s = s_vec(s_cnt + 1);

        Theta0 = [-s a_0; -b_0 s] ./ epsi;
        expmH_matrix = zeros(2, 2, N_grid);

        for k = 0:(N_grid - 1)
            H_x = diag([exp(-0.5*gamma*xspan(k + 1)); exp(0.5*gamma*xspan(k + 1))]);
            expmH_matrix(:, :, k + 1) = expm(-(Theta0 + 0.5 * gamma * V) * xspan(k + 1)) * H_x;
        end;

        P_matrix = zeros(2, 2, N_grid);
        for k = 0:(N_grid - 1)
            P_matrix(:, :, k + 1) = expmH_matrix(:, :, k + 1) \ expmH_matrix(:, :, N_grid);
        end;

        Q_matrix = zeros(2, N_grid);
        for k = 0:(N_grid - 2)
            dummy1 = 0.5 * (expmH_matrix(:, :, k + 1) * Upsilon(:, k + 1) ...
                         + (expmH_matrix(:, :, N_grid)) * Upsilon(:, N_grid));
            for kk = (k + 1):(N_grid - 2)
                dummy1 = dummy1 + (expmH_matrix(:, :, kk + 1)) * Upsilon(:, kk + 1);
            end;
            Q_matrix(:, k + 1) = Delta_grid * (expmH_matrix(:, :, k + 1) \ dummy1);
        end;

        M_matrix = P_matrix(:, :, 1);
        N_matrix = Q_matrix(:, 1);

        D = (s * eye(length(A)) - A) \ (expm(A / epsi) * L);
        g = C * D;
        varphi = mu.' * M_matrix / (g + mu'*(M_matrix(:, 1) + N_matrix));

        H_Xhat_YU = D * varphi;

        h_Xhat_Y = H_Xhat_YU(:, 1);
        h_Xhat_U = H_Xhat_YU(:, 2);
        
        
        h_uY = zeros(N_grid, 1);
        h_vY = zeros(N_grid, 1);
        h_uU = zeros(N_grid, 1);
        h_vU = zeros(N_grid, 1);
        
        for k = 1:N_grid
            H_w_YU = P_matrix(:, :, k) - (Q_matrix(:, k) + P_matrix(:, 1, k)) * varphi;
            
            h_uY(k) = H_w_YU(1, 1);
            h_vY(k) = H_w_YU(2, 1);
            h_uU(k) = H_w_YU(1, 2);
            h_vU(k) = H_w_YU(2, 2);
        end;
        
        h_aY = zeros(N_grid, 1);
        h_bY = zeros(N_grid, 1);
        h_aU = zeros(N_grid, 1);
        h_bU = zeros(N_grid, 1);
        
        h_aY(1) = h_uY(1);
        h_aU(1) = h_uU(1);
        h_bY(1) = h_vY(1);
        h_bU(1) = h_vU(1);
        
        for k = 2:N_grid
            h_aY(k) = h_uY(k) - Delta_grid * [(0.5 * Kuu(1, k)); Kuu(2:(k - 1), k); (0.5 * Kuu(k, k))]' * h_uY(1:k) ...
                              - Delta_grid * [(0.5 * Kuv(1, k)); Kuv(2:(k - 1), k); (0.5 * Kuv(k, k))]' * h_vY(1:k);  
                          
            h_aU(k) = h_uU(k) - Delta_grid * [(0.5 * Kuu(1, k)); Kuu(2:(k - 1), k); (0.5 * Kuu(k, k))]' * h_uU(1:k) ...
                              - Delta_grid * [(0.5 * Kuv(1, k)); Kuv(2:(k - 1), k); (0.5 * Kuv(k, k))]' * h_vU(1:k);
                          
            h_bY(k) = h_vY(k) - Delta_grid * [(0.5 * Kvu(1, k)); Kvu(2:(k - 1), k); (0.5 * Kvu(k, k))]' * h_uY(1:k) ...
                              - Delta_grid * [(0.5 * Kvv(1, k)); Kvv(2:(k - 1), k); (0.5 * Kvv(k, k))]' * h_vY(1:k);  
                          
            h_bU(k) = h_vU(k) - Delta_grid * [(0.5 * Kvu(1, k)); Kvu(2:(k - 1), k); (0.5 * Kvu(k, k))]' * h_uU(1:k) ...
                              - Delta_grid * [(0.5 * Kvv(1, k)); Kvv(2:(k - 1), k); (0.5 * Kvv(k, k))]' * h_vU(1:k);  
        end;
        
        h_deltaY = zeros(N_grid, 1);
        h_deltaU = zeros(N_grid, 1);
        
        
        for k = 1:N_grid
            xi = Delta_grid * k;
            
            if (xi + x_bar >= 1)
                h_deltaY(k) = interp1(xspan, h_aY, xi + x_bar - 1, 'linear');
                h_deltaU(k) = interp1(xspan, h_aU, xi + x_bar - 1, 'linear');
            else
                h_deltaY(k) = q * interp1(xspan, h_bY, 1 - xi - x_bar, 'linear');
                h_deltaU(k) = q * interp1(xspan, h_bU, 1 - xi - x_bar, 'linear');
            end;
        end;
        
        h_VY_int1 = La_x_bar(1:x_bar_ind)' * h_deltaY(1:x_bar_ind);
        h_VY_int2 = quadv(@(xi) (interp1(xispan, Lb_x_bar, xi, 'linear') * interp1(xspan, h_bY, 1 + xi - x_bar, 'linear')), 0, x_bar);
        
        h_VY = (1 / r) * K_psf_x_bar * h_Xhat_Y + (1 / r) * h_deltaY(x_bar_ind) + h_VY_int1 + h_VY_int2;
        
        h_VU_int1 = La_x_bar(1:x_bar_ind)' * h_deltaU(1:x_bar_ind);
        h_VU_int2 = quadv(@(xi) (interp1(xispan, Lb_x_bar, xi, 'linear') * interp1(xspan, h_bU, 1 + xi - x_bar, 'linear')), 0, x_bar);
        h_VU = (1 / r) * K_psf_x_bar * h_Xhat_U + (1 / r) * h_deltaU(x_bar_ind) + h_VU_int1 + h_VU_int2;


        num_integral = Delta_grid * [(0.5 * Kvu_x_bar(1)); Kvu_x_bar(2:(N_grid-1)); (0.5 * Kvu_x_bar(N_grid))]' * h_uY + ...
                       Delta_grid * [(0.5 * Kvv_x_bar(1)); Kvv_x_bar(2:(N_grid-1)); (0.5 * Kvv_x_bar(N_grid))]' * h_vY;

        den_integral = Delta_grid * [(0.5 * Kvu_x_bar(1)); Kvu_x_bar(2:(N_grid-1)); (0.5 * Kvu_x_bar(N_grid))]' * h_uU + ...
                       Delta_grid * [(0.5 * Kvv_x_bar(1)); Kvv_x_bar(2:(N_grid-1)); (0.5 * Kvv_x_bar(N_grid))]' * h_vU;        
        
        h_YU_psf(s_cnt + 1) = (num_integral + h_VY) / (1 - den_integral - h_VU);
    end;