function [h_YU_rec, h_YU_inf] = h_YU_sampling(s_vec, hypSys)
    N_grid = hypSys.N_grid;
    Delta_grid = hypSys.Delta_grid;
    
    r = hypSys.r;
    q = hypSys.q;
    x_bar = hypSys.x_bar;
    a_0 = hypSys.a_0;
    b_0 = hypSys.b_0;
    epsi = hypSys.epsi;
    gamma = hypSys.gamma;
    A = hypSys.A;
    C = hypSys.C;
    L = hypSys.obsv_params.L;
    
    p1 = hypSys.obsv_params.p1;
    p2 = hypSys.obsv_params.p2;

    K_inf_x_bar = hypSys.ctrl_params.K_inf_x_bar;
    K_rec_x_bar = hypSys.ctrl_params.K_rec_x_bar;

    x_bar_index = 1 + ceil(x_bar / Delta_grid);

    Kvu_x_bar = hypSys.Kvu(:, x_bar_index);
    Kvv_x_bar = hypSys.Kvv(:, x_bar_index);

    La_x_bar = hypSys.La(1:x_bar_index, x_bar_index);
    Lb_x_bar = hypSys.Lb(1:x_bar_index, x_bar_index);

    xspan = (0:Delta_grid:1)';
    xispan = (0:(x_bar_index - 1))' * Delta_grid;


    s_num = length(s_vec);

    h_YU_rec = zeros(s_num, 1);
    h_YU_inf = zeros(s_num, 1);

    V = [-1 0; 0 1];
    mu = [1; -q];

    Upsilon = zeros(2, N_grid);
    for k = 0:(N_grid - 1)
        Upsilon(:, k + 1) = (1 / epsi) * [p1(k + 1); -p2(k + 1)];
    end;

    for s_cnt = 0:(s_num - 1)
        s = s_vec(s_cnt + 1);

        Theta0 = [-s a_0; -b_0 s] ./ epsi;
        expmH_matrix = zeros(2, 2, N_grid);

        for k = 0:(N_grid - 1)
            H_x = diag([exp(-0.5*gamma*xspan(k + 1)); exp(0.5*gamma*xspan(k + 1))]);
            expmH_matrix(:, :, k + 1) = expm(-(Theta0 + 0.5 * gamma * V) * xspan(k + 1)) * H_x;
        end;

        P_matrix = zeros(2, 2, N_grid);
        for k = 0:(N_grid - 1)
            P_matrix(:, :, k + 1) = expmH_matrix(:, :, k + 1) \ expmH_matrix(:, :, N_grid);
        end;

        Q_matrix = zeros(2, N_grid);
        for k = 0:(N_grid - 2)
            dummy1 = 0.5 * (expmH_matrix(:, :, k + 1) * Upsilon(:, k + 1) ...
                         + (expmH_matrix(:, :, N_grid)) * Upsilon(:, N_grid));
            for kk = (k + 1):(N_grid - 2)
                dummy1 = dummy1 + (expmH_matrix(:, :, kk + 1)) * Upsilon(:, kk + 1);
            end;
            Q_matrix(:, k + 1) = Delta_grid * (expmH_matrix(:, :, k + 1) \ dummy1);
        end;

        M_matrix = P_matrix(:, :, 1);
        N_matrix = Q_matrix(:, 1);

        D = (s * eye(length(A)) - A) \ (expm(A / epsi) * L);
        g = C * D;
        varphi = mu.' * M_matrix / (g + mu'*(M_matrix(:, 1) + N_matrix));

        H_Xhat_YU = D * varphi;

        h_Xhat_Y = H_Xhat_YU(:, 1);
        h_Xhat_U = H_Xhat_YU(:, 2);
        
        
        h_uY = zeros(N_grid, 1);
        h_vY = zeros(N_grid, 1);
        h_uU = zeros(N_grid, 1);
        h_vU = zeros(N_grid, 1);
        
        for k = 0:(N_grid - 1)
            H_w_YU = P_matrix(:, :, k + 1) - (Q_matrix(:, k + 1) + P_matrix(:, 1, k + 1)) * varphi;
            
            h_uY(k + 1) = H_w_YU(1, 1);
            h_vY(k + 1) = H_w_YU(2, 1);
            h_uU(k + 1) = H_w_YU(1, 2);
            h_vU(k + 1) = H_w_YU(2, 2);
        end;

        num_integral = Delta_grid * [(0.5 * Kvu_x_bar(1)); Kvu_x_bar(2:(N_grid-1)); (0.5 * Kvu_x_bar(N_grid))]' * h_uY + ...
                       Delta_grid * [(0.5 * Kvv_x_bar(1)); Kvv_x_bar(2:(N_grid-1)); (0.5 * Kvv_x_bar(N_grid))]' * h_vY;

        den_integral = Delta_grid * [(0.5 * Kvu_x_bar(1)); Kvu_x_bar(2:(N_grid-1)); (0.5 * Kvu_x_bar(N_grid))]' * h_uU + ...
                       Delta_grid * [(0.5 * Kvv_x_bar(1)); Kvv_x_bar(2:(N_grid-1)); (0.5 * Kvv_x_bar(N_grid))]' * h_vU;


        den1_rec_integral = quadv(@(xi) (interp1(xispan, La_x_bar, xi, 'linear') * exp(-s*(x_bar + xi) / epsi)), 0, x_bar);
        den2_rec_integral = quadv(@(xi) (interp1(xispan, Lb_x_bar, xi, 'linear') * exp(-s*(x_bar - xi) / epsi)), 0, x_bar);


        g_rec = 1 / (r - q * exp(-s * 2 * x_bar / epsi) - q * den1_rec_integral - den2_rec_integral);

        h_YU_rec(s_cnt + 1) = (num_integral + g_rec * K_rec_x_bar * h_Xhat_Y) ...
                            / (1 - (den_integral + g_rec * K_rec_x_bar * h_Xhat_U));


        h_YU_inf(s_cnt + 1) = (num_integral + (1 / r) * K_inf_x_bar * h_Xhat_Y) ...
                                / (1 - den_integral - (1 / r) * K_inf_x_bar * h_Xhat_U);
    end;