function hypSys = hypSys_from_well(N, N_tr, x_bar, ctrl_on_limit, obsv_on_limit, poles, well_data)
    
    %% Extraction of well parameters
    beta    = well_data.beta;
    rho     = well_data.rho;
    l       = well_data.l;
%     A_1     = well_data.A_1;
    A_2     = well_data.A_2;
    A_bar   = well_data.A_bar;
    C_bar   = well_data.C_bar;
    F_1     = well_data.F_1;
    g       = well_data.g;

    %% Derived hypSys parameters
    epsi = sqrt(beta/rho)/l;
    gamma = l*F_1/sqrt(beta*rho);

    c_0 = -0.5*(F_1/rho);
    
    a_0 = c_0 * exp(-gamma*x_bar);
    b_0 = c_0 * exp( gamma*x_bar);

    % c = c_0/epsi;

    q = - exp(-gamma*x_bar);
%     q = - 10 * exp(-gamma*x_bar);
%     q = - 0.01 * exp(-gamma*x_bar);
    
    Delta = 1/(N - 1);
    
    %% Control objective
    r = 1;
    p_sp = 250e5 + l * (1 - x_bar) * 1e4;
    
    %% Disturbance model
    A = A_bar;
    C = - exp(-0.5*gamma*x_bar) * A_2 * C_bar;
    
    M = length(C);
    
    
    %% Transmission line
    L_tr = 2/epsi;
    Delta_tr = L_tr/(N_tr + 1);

    
    %% Kernels
    N_grid = N + 2;
    
    Delta_grid = 1/(N_grid - 1);
        
    numIter         = 10;    % Adjustable
    
    [Kuu, Kuv, Kvu, Kvv] = K_kernel_solver(N_grid, a_0 / epsi, b_0 / epsi, gamma, q, numIter);

    [Laa, Lab, Lba, Lbb] = L_kernel_solver(N_grid, a_0 / epsi, b_0 / epsi, gamma, q, numIter);
    
    [Puu, Puv, Pvu, Pvv] = P_kernel_solver(N_grid, a_0 / epsi, b_0 / epsi, gamma, q, numIter);

    La  = Laa - r*Lba;
    Lb  = Lab - r*Lbb;
    
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Store values
    hypSys.well_data    = well_data;
    hypSys.N            = N;
    hypSys.x_bar        = x_bar;
    
    hypSys.epsi         = epsi;
    hypSys.gamma        = gamma;
    hypSys.c_0          = c_0;
    hypSys.a_0          = a_0;
    hypSys.b_0          = b_0;
    hypSys.q            = q;
    hypSys.Delta        = Delta;
    
    hypSys.r            = r;
    hypSys.p_sp         = p_sp;
    
    hypSys.A            = A;
    hypSys.C            = C;
    hypSys.M            = M;
   
    hypSys.N_tr         = N_tr;
    hypSys.L_tr         = L_tr;
    hypSys.Delta_tr     = Delta_tr;
    
    hypSys.N_grid       = N_grid;
    hypSys.Delta_grid   = Delta_grid;
    
    hypSys.Kuu          = Kuu;
    hypSys.Kuv          = Kuv;
    hypSys.Kvu          = Kvu;
    hypSys.Kvv          = Kvv;
    
    hypSys.La           = La;
    hypSys.Lb           = Lb;
    
    hypSys.Puu          = Puu;
    hypSys.Puv          = Puv;
    hypSys.Pvu          = Pvu;
    hypSys.Pvv          = Pvv;
    
    hypSys.ctrl_on_limit = ctrl_on_limit;
    hypSys.obsv_on_limit = obsv_on_limit;
    
    hypSys.ctrl_params = controller_gains(hypSys);
    
    hypSys.obsv_params = observer_gains(poles, hypSys);