%% Parameter extraction 
beta = hypSys.well_data.beta; % Pascal
rho = hypSys.well_data.rho; % kg/m3
l = hypSys.well_data.l; % m
A_1 = hypSys.well_data.A_1; % m2
F_1 = hypSys.well_data.F_1;% 0.0158/l; % ?
epsi = hypSys.epsi;
gamma = hypSys.gamma;
q = hypSys.q;
A = hypSys.A;
C = hypSys.C;
x_bar = hypSys.x_bar;

g = 9.81;
p_sp = hypSys.p_sp;

M = hypSys.M;

N = hypSys.N;
N_grid = hypSys.N_grid;
N_tr = hypSys.N_tr;

Delta_grid = hypSys.Delta_grid;

%% Variable extraction
u_log = x_log(:, 1:N);
v_log = x_log(:, (N + 1):(2*N));
X_log = x_log(:, (2*N + 1):(2*N + M));

z_log = x_log(:, (2*N + M + 1):(2*N + M + N_tr));

%% Variable augmentation
    % System variables
u_aug = [(2*u_log(:, 1) - u_log(:, 2)) ...
               u_log(:, :) ...
            (2*u_log(:, N) - u_log(:, N - 1))];
v_aug = [(2*v_log(:, 1) - v_log(:, 2)) ...
               v_log(:, :) ...
            (2*v_log(:, N) - v_log(:, N - 1))];
         
    % Transmission line
z_aug = [2*z_log(:, 1) - z_log(:, 2) ...
              z_log(:, :) ...
            2*z_log(:, N_tr) - z_log(:, N_tr - 1)];


%% Computed values
numT = length(Tspan);

xspan = 0:Delta_grid:1;
XSPAN = repmat(xspan, numT, 1);
[X_arr, T_arr] = meshgrid(xspan, Tspan);
P =  (  u_aug.*exp(-0.5.*gamma.*(XSPAN - x_bar)) ...
      - v_aug.*exp( 0.5.*gamma.*(XSPAN - x_bar))) .* sqrt(beta*rho)/A_1 ...
      + p_sp - rho * g * l * (XSPAN - x_bar);

indexOn = (hypSys.ctrl_on_limit * 10);


U = z_aug(:, 1)' + hypSys.ctrl_params.ku * u_aug' ...
                 + hypSys.ctrl_params.kv * v_aug';
                      
U(1:indexOn) = zeros(indexOn, 1);
       


% Dimensions
figBx = 640;
figBy = 512;
figBigBy = figBy*2;


%% Figure 1: p(x_bar*l, t)
figure(1);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
    subplot(3, 1, 1);    
    exact_index = 1 + (N_grid - 1)*x_bar;
    allowed_deviation = 0.05;
    [i, r] = deal(fix(exact_index), exact_index-fix(exact_index));
    
    if ((r > allowed_deviation) && (r < (1 - allowed_deviation)))
        p_xbar = (1 - r)*P(:, i) + r * P(:, i + 1);
    else
        p_xbar = P(:, i);
    end;
    
    
    plot(Tspan, ones(numT, 1)*p_sp*1e-5, 'color', [0 0.5 0], 'LineWidth', 2);
    hold on;
    plot(Tspan, p_xbar*1e-5, 'color', [0 0 0], 'LineWidth', 2);
    hold off;
    grid on;
    
    V = [0 Tspan(end) (p_sp*1e-5 - 100) (p_sp*1e-5 + 100)];
    axis(V);
    
    xlabel('t [sec]');
    ylabel('Pressure [bar]');
    title('Pressure at z = z_{bar}');

    legend('Setpoint', 'Actual', 'Location', 'Best');
    

    
startIndexZoom1 = 401;
zoom1 = 10;
%% Figure 2: p(x_bar*l, t), zoom 1
figure(2);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
    subplot(3, 1, 1);
    plot(Tspan(startIndexZoom1:numT), ones(length(startIndexZoom1:numT), 1)*p_sp*1e-5, 'color', [0 0.5 0], 'LineWidth', 2);
    hold on;
    plot(Tspan(startIndexZoom1:numT), p_xbar(startIndexZoom1:numT)*1e-5, '--', ...
            'color', [0 0 0], 'LineWidth', 2);
    hold off;
    grid on;

    V = [0 Tspan(end) (p_sp*1e-5 - zoom1) (p_sp*1e-5 + zoom1)];
    axis(V);

    xlabel('t [sec]');
    ylabel('Pressure [bar]');
    title('Pressure at z = z_{bar} (zoom 1)');

    legend('Setpoint', 'Actual', 'Location', 'Best');
    

startIndexZoom2 = 401;
zoom2 = 0.6;
%% Figure 3: p(x_bar*l, t), zoom 2
figure(3);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
    subplot(3, 1, 1);
    plot(Tspan(startIndexZoom2:numT), ones(length(startIndexZoom2:numT), 1)*p_sp*1e-5, 'color', [0 0.5 0], 'LineWidth', 2);
    hold on;
    plot(Tspan(startIndexZoom2:numT), p_xbar(startIndexZoom2:numT)*1e-5, '--', ...
            'color', [0 0 0], 'LineWidth', 2);
    hold off;
    grid on;

    V = [0 Tspan(end) (p_sp*1e-5 - zoom2) (p_sp*1e-5 + zoom2)];
    axis(V);

    xlabel('t [sec]');
    ylabel('Pressure [bar]');
    title('Pressure at z = z_{bar} (zoom 2)');

    legend('Setpoint', 'Actual', 'Location', 'Best');
    


%% Figure 4: U(t), state feedback
figure(4);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
    subplot(3, 1, 1);
    plot(Tspan, U, 'k', 'LineWidth', 2);
%     V = [0 Tspan(end) -0.2 0.2];
%     axis(V);
    grid on;
    xlabel('t [sec]');
    ylabel('U(t) [m^3/s]');
    title('Control signal');
    



%% Figure 6: Pressure time series at selected depths, state feedback
figure(6);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);

subplot(2, 1, 1);

colorVals = [[1 0 0]; [0 1 0]; [0 0 1]; [1 0.5 0]; [1 0 1]; [0 0 0]];
spacing = 500;
for k = 0:5
    thePlot  = plot(Tspan, P(:, 1 + round((N_grid - 1)*(spacing*k/l)))*1e-5);
    set(thePlot, 'color', colorVals(k + 1, :));
    grid on;
    hold on;
end; % 242


V = [0 Tspan(end) 0 800];
axis(V);

xlabel('t [sec]');
ylabel('Pressure [bar]');
title('Pressure distribution at selected well depths');

legend('BHP', '2000 m', '1500 m', '1000 m', '500 m', 'Topside', 'Location', 'Best');
hold off;



%% Figure 8: p
figure(8);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
    % p
    subplot(2, 1, 1);
    mesh(T_arr, l*(1 - X_arr), P*1e-5);
    hold on;
    plot3(Tspan, ones(size(Tspan))*l*(1 - x_bar), P(:, 1 + round((N_grid - 1)*x_bar))*1e-5, ...
            'color', [0 0 0], 'LineWidth', 2);
    hold off;
    
    V = [0 Tspan(end) 0 l 0 700];
    axis(V);
    
    view([35 25]);
    xlabel('t [sec]');
    ylabel('Depth [m]');
    zlabel('p [bar]');
    title('Pressure distribution');
        

%     %% Figure 12: p_l(t)
%     figure(12);
%     set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);
%         subplot(3, 1, 1);
% 
%         plot(Tspan, P(:, end)*1e-5, 'color', [0 0 0], 'LineWidth', 2);
%         grid on;
% 
%         V = [0 Tspan(end) (P(1, end)*1e-5 - 150) (P(1, end)*1e-5 + 150)];
%         axis(V);
% 
%         xlabel('t [sec]');
%         ylabel('Pressure [bar]');
%     %     title('Pressure at point of interest');



%% Figure 9: Well pressure profiles
figure(9);
set(gcf, 'Color', [1 1 1], 'Units', 'pixels', 'Position', [0,0,figBx,figBigBy]);

numOfLines = 4;
numPlots = 2;
spacing = 10; % Number of samples between each line
% V = [0 l min(min(P_sf*1e-5)) max(max(P_sf*1e-5))];
V = [0 l 100 700];

inds = 40 + (0:3) * 4;
subPlotIndecing = [1 2];

% inds = 0 + (0:7) * 5;
% subPlotIndecing = [1 3 5 7 2 4 6 8];


for i = 1:numPlots
    subplot(3, 2, subPlotIndecing(i));
    
    curLines = zeros(numOfLines, 1);
    for k = 0:(numOfLines - 1)
%         colorVal = [0.0 0.0 1] + 0.8*([1 1 0] * ((numOfLines - k - 1)/(numOfLines - 1))).^1;
        colorVal = [0.0 0.0 0.0] + 0.8*([1 1 1] * ((numOfLines - k - 1)/(numOfLines - 1))).^1;
        if (k == numOfLines - 1)
            curLines(k + 1) = plot(l*xspan, fliplr(P(1 + (1/h) * inds(i) + spacing * k, :))*1e-5, 'Color', colorVal);
        else
            curLines(k + 1) = plot(l*xspan, fliplr(P(1 + (1/h) * inds(i) + spacing * k, :))*1e-5, '--', 'Color', colorVal);
        end;
        axis(V);
        grid on;
        hold on; 
    end;
    lineGroup = hggroup;
    set(curLines,'Parent',lineGroup);
    
    set(get(get(lineGroup,'Annotation'),'LegendInformation'),...
    'IconDisplayStyle','on'); 

    plot(l*(1 - x_bar), p_sp*1e-5, 'r*'); 
    xlabel('Depth [m]');
    ylabel('Pressure [bar]');
    title(['From t = ' num2str(inds(i)) ' sec']);

%     y_legend = legend('Pressures', 'Setpoint', 'Location', 'best');
%     set(y_legend, 'Fontsize', 12, 'interpreter', 'latex');
%     set(y_legend, 'Fontsize', 12);

    legend('Pressures', 'Setpoint', 'Location', 'Best');
    
	hold off;
end;